unit NaKpump;
{ Smith and Crampin // Biophisics and Mol. Biol. 2004 }

interface
uses MathMyO;

procedure Initiate_NaKpump_SmithCrampin;
function NaKpump_SmithCrampin(V_m{V}, Na_e{mM}, Na_i{mM}, K_e{mM}, K_i{mM} :double) :double;

implementation


var
    T,R,F, MgATP,MgADP, SPi,H,G_MgATP_0,
    Kd_HPi,Kd_KPi,Kd_NaPi,k_p_1,k_m_1,k_p_2,k_m_2,k_p_3,k_m_3,k_p_4,k_m_4,
    Kd_0_Nae,Kd_0_Nai,Kd_Ke,Kd_Ki,Kd_MgATP,delta,v_cyc_s
                                                    :double;


procedure Initiate_NaKpump_SmithCrampin;
begin
  { Parameters }
  T := 310;  {K}
  R := 8.314; {J/mol}
  F := 96485; {C/mol}
  MgATP := 9.8;
  MgADP := 0.05;
  SPi := 4.2;
  H := 1e-7;
  G_MgATP_0 := 11900;
  //'dissociation constants for inorganic phosphate, mM'
  Kd_HPi := 6.77;
  Kd_KPi := 292;
  Kd_NaPi := 224;
  //'parameter - fitted value'
  k_p_1 := 1050; // s(-1)
  k_m_1 := 172.1; // s(-1)mM(-1)
  k_p_2 := 481; // s(-1)
  k_m_2 := 40; // s(-1)
  k_p_3 := 2000; // s(-1)
  k_m_3 := 79300; // s(-1)mM(-2)
  k_p_4 := 320; // s(-1)
  k_m_4 := 40; // s(-1)
  Kd_0_Nae := 15.5; // mM
  Kd_0_Nai := 2.49; // mM
  Kd_Ke := 0.213;
  Kd_Ki := 0.500;
  Kd_MgATP := 2.51;
  delta := -0.031;
  //'normalization'
  v_cyc_s :=  27.338758122549248;
end;

function NaKpump_SmithCrampin(V_m{V}, Na_e{mM}, Na_i{mM}, K_e{mM}, K_i{mM} :double) :double;
var
    Kd_Nae,Kd_Nai,Na_t_i,K_t_i,Na_t_e,K_t_e,MgATP_t,Pi,
    a_1,a_2,a_3,a_4, b_1,b_2,b_3,b_4,s,v_cyc
                                                                      :double;
begin
    //'SMITH CRAMPIN'
    //'vlotage dependance of the sodium dissociation reaction  (14)'

    Kd_Nae := Kd_0_Nae * dexp(((1 + delta) * F * V_m )/ (3 * R * T));
    Kd_Nai := Kd_0_Nai * dexp((delta * F * V_m )/ (3 * R * T));

    //'parametrs  (32)'
    Na_t_i := Na_i / Kd_Nai;
    K_t_i  := K_i / Kd_Ki;
    Na_t_e := Na_e / Kd_Nae;
    K_t_e  := K_e / Kd_Ke;
    MgATP_t := MgATP / Kd_MgATP;

    //'SPi - mesurable concentration, HPi, KPi, NaPi - bound species  (33)'

    Pi := SPi / (1 + (K_i / Kd_KPi) + (H / Kd_HPi) + (Na_i / Kd_NaPi));

    //'transition rates for forward reaction steps  (30)'

    a_1 := k_p_1 * istep(Na_t_i,3) / (istep(1 + Na_t_i,3) + sqr(1 + K_t_i) - 1);
    a_2 := k_p_2;
    a_3 := k_p_3 * sqr(K_t_e)      / (istep(1 + Na_t_e,3) + sqr(1 + K_t_e) - 1);
    a_4 := k_p_4 * MgATP_t / (1 + MgATP_t);

    //'transition rates for reverse reaction steps  (31)'

    b_1 := k_m_1 * MgADP;
    b_2 := k_m_2 * istep(Na_t_e,3) / (istep(1 + Na_t_e,3) + sqr(1 + K_t_e) -1);
    b_3 := k_m_3 * Pi * H / (1 + MgATP_t);
    b_4 := k_m_4 * sqr(K_t_i)      / (istep(1 + Na_t_i,3) + sqr(1 + K_t_i) -1);

    //'16 positive terms, diagram method'
    s:= a_1 * a_2 * a_3 + a_4 * a_1 * a_2 + a_3 * a_4 * a_1 + a_2 * a_3 * a_4 +
        b_1 * b_4 * b_3 + b_2 * b_1 * b_4 + b_4 * b_3 * b_2 + b_3 * b_2 * b_1 +
        a_4 * b_2 * b_1 + a_4 * b_2 * a_1 + b_4 * a_2 * a_3 + b_4 * a_2 * b_3 +
        a_1 * a_2 * b_3 + a_1 * b_3 * b_2 + b_1 * b_4 * a_3 + b_1 * a_4 * a_3;
    //'clock-wise rate at steady-state  (34)'
    v_cyc := ( a_1 * a_2 * a_3 * a_4 + b_1 * b_2 * b_3 * b_4) / (s * v_cyc_s); {1/s}
    NaKpump_SmithCrampin:=v_cyc;
end;

end.
 